import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import japanize_matplotlib


#   ExcelファイルをPandasのDataFrame型として読み込む
fl_nm = input('Excel Data File(*.xlsx) = ')
xlfile = pd.ExcelFile(fl_nm)
df = pd.read_excel(xlfile)

#    １行目の値（列変数名）
col_names = df.keys()         #    Excelの１行目
c00 = col_names[0]            #    行変数名の列のキー名
col_names = col_names[1:]     #    ２番目以降を取り出す
print('col_names...\n', col_names)

#    行変数名
row_names = df[c00]
print('row_names...\n', row_names)

#    セル内の数値
df_data = df.values              #   ２行目以降が取り出される
F = df_data[:,1:]                #    取り出した各行の２列目以降を取り出す

data = np.asfarray(F)

#     dataの特異値分解SVD
u, s, vh = np.linalg.svd(data, full_matrices = False)

print('data =\n', data)

p = sum([1 if v > 1.0E-9 else 0 for v in s])
s = s[:p]
print('\ns =', s)
print('p =', p)
u = u[:,:p]   # (u.T[:p]).T
vh = vh[:p]

plt.title('Scree Plot of Singular Values', fontsize=14)
plt.plot(range(1, p+1), s, c = 'k')
plt.xticks(np.arange(1, p+1, 1))
plt.ylabel('Value', fontsize = 14)
plt.tight_layout()
plt.savefig('Fig_Singular.png')
plt.show()
print('\n', 'Fig_Singular.png', 'was saved.')


M = len(data)
N = len(data.T)
s1 = np.empty(p)
s2 = np.empty(p)
for t in range(p):
    s1[t] = (s[t]**0.5) * ((M/N)**0.25)  
    s2[t] = (s[t]**0.5) * ((N/M)**0.25)  

#     特異値による重み付け
row_coord = u @ np.diag(s1)
col_coord = np.diag(s2) @ vh

#
#        行情報と列情報の積み重ね
#
mrg_coord = np.concatenate((row_coord, col_coord.T), axis = 0)
mrg_label = np.concatenate((row_names, col_names), axis = 0)

#
#        積み重ね座標の中心化と主成分分析（標準偏差の調整無し）
#
mean_X = np.mean(mrg_coord, axis = 0)
Z = mrg_coord - mean_X
w, v = np.linalg.eigh(Z.T @ Z)

sidx = np.argsort(-w)
w = w[sidx]
v = v.T[sidx].T
print('w =\n', w)
plt.title('Scree Plot of w', fontsize=16)
plt.plot(np.arange(len(w))+1,w)
plt.ylabel('Eigen value', fontsize=14)
plt.tight_layout()
plt.savefig('Fig_Eigen.png')
plt.show()
print('\n', 'Fig_Eigen.png', 'was saved.')
#
#        座標軸の回転
#
Proj = v.T 
Proj_mrg_coord = (Proj @ mrg_coord.T).T

p_dim = 5 if p > 5 else p
fout = open('Coord.txt', 'w')
fout.write(f'\n\n{"Names":<15s}')
for i in range(p_dim):
    fout.write(f'{f"Dim-{i+1}":<15s}')
fout.write('\n')
for i in range(M):
    fout.write(f'{row_names[i]:<15s}')
    for j in range(p_dim):
        fout.write(f'{Proj_mrg_coord[i][j]:<15.5f}')
    fout.write('\n')

fout.write('\n')
for i in range(N):
    fout.write(f'{col_names[i]:<15s}')
    for j in range(p_dim):
        fout.write(f'{Proj_mrg_coord[M+i][j]:<15.5f}')
    fout.write('\n')

fout.close()
print('\n', 'Coord.txt', 'was saved.')

#
#        布置の描画
#
max_coord = np.max(np.abs(Proj_mrg_coord))

dimx = 0
dimy = 1
while True:
    plt.figure(figsize=(7,7))   # figsize = (5, 4))
    plt.xticks([-max_coord, max_coord])
    plt.yticks([-max_coord, max_coord])
    plt.plot(Proj_mrg_coord.T[dimx][:M], Proj_mrg_coord.T[dimy][:M], '^', c = 'g', label = 'Row')
    plt.plot(Proj_mrg_coord.T[dimx][M:], Proj_mrg_coord.T[dimy][M:], 'v', c = 'b', label = 'Col') 
    for i in range(M):
        plt.text(Proj_mrg_coord.T[dimx][i], Proj_mrg_coord.T[dimy][i] - 0.05,
                 row_names[i], rotation = 0, fontsize = 14, color='g') 
    for i in range(N):
        plt.text(Proj_mrg_coord.T[dimx][M + i], Proj_mrg_coord.T[dimy][M + i] + 0.05,
                 col_names[i], rotation = 0, fontsize = 14, color='b')
    plt.plot([0], [0], '*', c = 'm', markersize = 12, label = 'Origin')
    plt.plot([-max_coord, max_coord], [0, 0], c = 'k', ls = ':')
    plt.plot([0, 0], [-max_coord, max_coord], c = 'k', ls = ':')
    plt.legend()#fontsize = 14)
    plt.title('Orthogonally Rotated Configulation', fontsize = 16)
    plt.xlabel(f'Axis-{dimx+1}', fontsize = 14)
    plt.ylabel(f'Axis-{dimy+1}', fontsize = 14)
    plt.tight_layout()
    plt.savefig(f'FigMapRotatedAixes_{dimx+1}_{dimy+1}.png')
    plt.show()
    print('\n', f'FigMapRotatedAixes_{dimx+1}_{dimy+1}.png', 'was saved.')

    print(f'\nMaximum dimension = {p}')
    dimx = int(input('Dimension for x-axis = '))
    if dimx < 1:
        break
    if dimx > p:
        dimx = p
    dimx -= 1
    dimy = int(input('Dimension for y-axis = '))
    if dimy < 1:
        break
    if dimy > p:
        dimy = p
    dimy -= 1
